use std::{cell::OnceCell, sync::OnceLock};

use lazy_static::lazy_static;
use nom::{
    bytes::complete::take_while1, character::complete::anychar, error::ErrorKind, multi::many_till,
    IResult,
};
use nom_regex::str::re_match;
use regex::Regex;

struct Sections {
    header: String,
}

static TIMESTAMP_REGEX: OnceLock<Regex> = OnceLock::new();
static START_BLOCK_REGEX: OnceLock<Regex> = OnceLock::new();
static END_BLOCK_REGEX: OnceLock<Regex> = OnceLock::new();
fn timestamp<'a>(input: &'a str) -> IResult<&'a str, &'a str> {
    re_match::<nom::error::Error<&str>>(TIMESTAMP_REGEX.get().unwrap().clone())(input)
}

fn debug_level(input: &str) -> IResult<&str, &str> {
    take_while1(|c: char| c.is_alphabetic() || ['_', '-', '%'].contains(&c))(input)
}

fn start_block(input: &str) -> IResult<&str, &str> {
    re_match::<nom::error::Error<&str>>(START_BLOCK_REGEX.get().unwrap().clone())(input)
}

fn end_block(input: &str) -> IResult<&str, &str> {
    re_match::<nom::error::Error<&str>>(END_BLOCK_REGEX.get().unwrap().clone())(input)
}


fn test_output_line(input: &str) -> IResult<&str, &str> {
    let (input, timestamp) = timestamp(input)?;
    let (input, _) = nom::character::complete::char(' ')(input)?;
    let (input, level) = debug_level(input)?;
    let (input, _) = nom::character::complete::char(' ')(input)?;
    let (input, _) = nom::character::complete::line_ending(input)?;
    let (input, msg) = message(input)?;
    IResult::Ok((input, ""))
}

fn header(input: &str) -> IResult<&str, String> {
    // any_char.until(timestamp).concat()
    let (_, (b, rest)) = many_till(anychar, timestamp)(input)?;
    let joined = b.iter().collect::<String>();
    // test_output_line.until(start_block)
    let (_, (c, rest)) = many_till(test_output_line, start_block)(rest)?;
    print!("{}", rest);
    IResult::Ok((input, joined))
}
fn sections(log: &str) -> IResult<&str, Sections> {
    let (_log, _header) = header(log)?;
    todo!()
}

fn main() {
    TIMESTAMP_REGEX
        .set(Regex::new(r"^\d{4}-\d{2}-\d{2}.\d{2}:\d{2}:\d{2}(\+\d{2}:\d{2})?(,\d{3})?").unwrap())
        .unwrap();

    START_BLOCK_REGEX
        .set(Regex::new(r"^.*\n^.*EASYPY JOB START.*$\n.*").unwrap())
        .unwrap();
    let file = include_str!("../consoleText");
    header(file);
}
